c
c=======================================================================
c  Include file "cstseg.h" for PE_initial.
c=======================================================================
#ifndef rmdocinc
c
c  CSMASK     mask for land P-points.
c  CSTSEGX    x-index coordinate for each coastal segment.
c  CSTSEGY    y-index coordinate for each coastal segment.
c  CVAL       values of transport function at coastal segments.
c  CVALIS     values of transport function at islands coastal segments.
c  CVALOPN    transport streamfunction at each point for the N-open
c             basin segment.
c  ICVALIS    Switch that indicates how to set transport streamfunction
c             around island N:
c             ICVALIS(n)=0  impose value found internally at the
c                           beginning of the island segment.
c             ICVALIS(n)=1  inpose value given in CVALIS(n).
c  IEIS       ending I-index for each island box.
# ifdef surfpress
c  IEP        Ending indices for surface pressure.     (integer array)
c  IEQ        Ending indices for velocity points.      (integer array)
# endif
c  ISIS       starting I-index for each island box.
# ifdef indv_isl
c  ISLIND     island indices sorted by coastal length.  (integer vector)
# endif
c  ISMASK     mask at the velocity points used for segment extraction.
# ifdef surfpress
c  ISP        Starting indices for surface pressure.   (integer array)
c  ISQ        Starting indices for velocity points.    (integer array)
# endif
c  JEIS       ending J-index for each island box.
c  JSIS       starting J-index for each island box.
c  LANDP      transport streamfunction land/sea mask. [0] sea, [other] cst #
c  LANDT      mask for tracer points: [0] land, [1] sea.
c  LANDV      mask for velocity points: [0] land, [1] boundary, [2] sea.
c  LENSEG     number of points in each coastal segment.
c  LSEGMX     current maximum number of points per coastal segment.
c  NISLE      number of islands in the model basin.
c  NOPNPS     number of open basin segments (excluding interior
c             islands) as specified in surface pressure BC file.
c  NOPNSEG    number of open basin segments (excluding interior
c             islands).
c  NPTPSOP    number of points in each N-open basin segment
c             as specified in surface pressure BC file.
c  NPTSOPN    number of points in each N-open basin segment.
c  NSEG       number of coastal segments.
c  OPENPS     Switch for handling surface pressure along the
c             N-open basin segments:
c             OPENPS=0      impose values found internally at the
c                           open basin segment (open boundary).
c             OPENPS=2      impose the values given in CVALOPN(n)
c                           which are prescribed in the file.
c  OPENSEG    Switch for handling transport streamfunction along the
c             N-open basin segments:
c             OPENSEG=0     impose values found internally at the
c                           open basin segment (open boundary).
c             OPENSEG=1     impose the values given in CVALOPN(n)
c                           at the coasts.
c             OPENSEG=2     impose the values given in CVALOPN(n)
c                           which are prescribed in the file.
c  PSVALOP    surface pressure at each point for the N-open
c             basin segment.
c  USEPSBY    Switch for using external surface pressure
c             boundary conditions in open domains.
c  USETRBY    Switch for using external transport streamfunction
c             boundary conditions in open domains.
c
c=======================================================================
#endif
c
      integer lsegmx,nisle,nopnps,nopnseg,nseg,openps,openseg
      integer csmask(np),cstsegx(mpseg,mcseg),cstsegy(mpseg,mcseg),
     &        icvalis(mcseg),ieis(misle),isis(misle),ismask(np),
     &        jeis(misle),jsis(misle),landp(np),landt(np),landv(np),
     &        lenseg(mcseg),nptpsop(mcseg),nptsopn(mcseg)
#ifdef surfpress
     &        ,iep(isesize),ieq(isesize),isp(isesize),isq(isesize)
#endif
#ifdef indv_isl
     &        ,islind(misle)
#endif
      logical usepsby,usetrby
c
      common /icstseg/ landp,landt,landv,ismask,nseg,lsegmx,lenseg,
     &                 cstsegx,cstsegy,nisle,isis,ieis,jsis,jeis,
     &                 icvalis,csmask,openps,openseg,nopnps,nopnseg,
     &                 nptpsop,nptsopn
#ifdef surfpress
     &                ,iep,ieq,isp,isq
#endif
#ifdef indv_isl
     &                ,islind
#endif
     &                ,usepsby,usetrby
c
      real cval(mcseg),cvalis(mcseg),cvalopn(mpseg,mcseg),
     &     psvalop(mpseg,mcseg)
c
      common /fcstseg/ cval,cvalis,cvalopn,psvalop
