      subroutine dt2nd (locale,ptr,ibdy,time,qbry,dtbry)
c
c=======================================================================
c                                                                    ===
c  This routine differentiates either the barotropic vorticity or a  ===
c  component of surface pressure gradient.                           ===
c                                                                    ===
c  ------                                                            ===
c  Input:                                                            ===
c  ------                                                            ===
c                                                                    ===
c     LOCALE   Location of derivative.              (string)         ===
c               "Left"   - valid at previous time.                   ===
c               "Center" - valid at current time.                    ===
c               "Right"  - valid at advanced time.                   ===
c               "Persist" - endpoint assuming persistence            ===
c                           beyond data.                             ===
c     PTR      Locations of specific times.         (integer vector) ===
c                 PTR(1) = previous time.                            ===
c                 PTR(2) = current time.                             ===
c                 PTR(3) = advanced time.                            ===
c     IBDY     Length of boundaries.                (integer vector) ===
c     TIME     Absolute times of boundary data.     (real vector)    ===
c     QBRY     Boundary data.                       (real array)     ===
c                                                                    ===
c  -------                                                           ===
c  Output:                                                           ===
c  -------                                                           ===
c                                                                    ===
c     DTBRY   Time derivative of boundary data.    (real array)      ===
c                                                                    ===
c  ------                                                            ===
c  Calls:                                                            ===
c  ------                                                            ===
c                                                                    ===
c     ALL_UC,  LENGTH                                                ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define global data.
c-----------------------------------------------------------------------
c
#include <param.h>
c
c-----------------------------------------------------------------------
c  Define local data.
c-----------------------------------------------------------------------
c
      real    c0,day2sec,p5
      parameter (c0=0.0, day2sec=86400.0, p5=0.5)
c
      integer m,n,nrm,sbn,sln,snd
      integer ibdy(4),ptr(3)
      real    dtm,dtp,wt,wta,wtb,sumdt
      real         dtbry(mx,4,3),qbry(mx,4,3,0:2),time(0:2)
      character*80  locstr
      character*(*) locale
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
c-----------------------------------------------------------------------
c  Set differentiating weights.
c-----------------------------------------------------------------------
c
c  Ensure case insensitivity.
c
      call all_uc (locale,locstr)
      call length (locstr,sln,sbn,snd)
c
c  Set weights by location.
c
      dtm   = (time(ptr(2)) - time(ptr(1)))*day2sec
      dtp   = (time(ptr(3)) - time(ptr(2)))*day2sec
      sumdt = dtp + dtm
c
      if (locstr(sbn:snd).eq.'CENTER') then
         wtb = -dtp / (dtm*sumdt)
         wt  = (dtp-dtm) / (dtm*dtp)
         wta = dtm / (dtp*sumdt)
c
       elseif (locstr(sbn:snd).eq.'LEFT') then
         wtb = -(dtm+sumdt) / (dtm*sumdt)
         wt  = sumdt / (dtm*dtp)
         wta = -dtm / (dtp*sumdt)
c
       elseif (locstr(sbn:snd).eq.'RIGHT') then
         wtb = dtp / (dtm*sumdt)
         wt  = -sumdt / (dtm*dtp)
         wta = (dtp+sumdt) / (dtp*sumdt)
c
       else
         wtb = c0
         wt  = -p5/dtp
         wta = p5/dtp
      endif
c
c-----------------------------------------------------------------------
c  Take second order time difference.
c-----------------------------------------------------------------------
c
      do 10 nrm = 1, 3
      do 10 n = 1, 4
      do 10 m = 1, ibdy(n)
         dtbry(m,n,nrm) = wta*qbry(m,n,nrm,ptr(3))
     &                    + wt*qbry(m,n,nrm,ptr(2))
     &                    + wtb*qbry(m,n,nrm,ptr(1))
  10  continue
c
      return
      end
