      subroutine get_tide (day)
c
c=======================================================================
c                                                                    ===
c  This routine gets the current tidal fields at the specified time. ===
c                                                                    ===
c  ------                                                            ===
c  Input:                                                            ===
c  ------                                                            ===
c                                                                    ===
c     DAY   Time at which tide is sought, relative to TSTART.  (days)===
c                                                                    ===
c  Common Blocks:                                                    ===
c                                                                    ===
c  /MODDAT/                                                          ===
c                                                                    ===
c     TIDTSTART  starting time for linear tidal model.   (days)      ===
c     TSTART     input day assigned for init/ass.        (days)      ===
c                                                                    ===
c  /NDIMEN/                                                          ===
c                                                                    ===
c     IM   number of points in the x-direction.                      ===
c     JM   number of points in the y-direction.                      ===
c                                                                    ===
c  /SWITCHES/                                                        ===
c                                                                    ===
c     TIDFLD   flag for use of linear tidal fields.                  ===
c              [0] do NOT use.                                       ===
c              [1] use with time referenced to TSTART                ===
c              [2] use with time referenced to TIDTSTART             ===
c                                                                    ===
c  -------                                                           ===
c  Output:                                                           ===
c  -------                                                           ===
c                                                                    ===
c  Common Blocks:                                                    ===
c                                                                    ===
c  /CURFLDS/                                                         ===
c                                                                    ===
c     DELVDT   Time rate of change of ELVTID.   (real array; cm/s)   ===
c     ELVTID   linear tidal surface elevation.  (real array; cm)     ===
c     UBTID    barotropic, linear tidal velocity, u-component.       ===
c     VBTID    barotropic, linear tidal velocity, v-component.       ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define global data.
c-----------------------------------------------------------------------
c
#include <cdefs.h>
#include <param.h>
#include <pconst.h>
#include <curflds.h>
#include <moddat.h>
#include <ndimen.h>
#include <switches.h>
c
c-----------------------------------------------------------------------
c  Define local data.
c-----------------------------------------------------------------------
c
      integer npcmp
      parameter (npcmp=np*maxcomp)
c
      integer cshft,imjm,ip,ipc,n,nocomp
      logical first
      FLOAT
     &      day,dstart,et,grav,ttsec,ut,vt
      FLOAT
     &      t0tid(maxcomp),tptid(maxcomp)
      COMPLEX
     &        cxtime,e_iomt
      COMPLEX
     &        iomega(maxcomp),iphase(maxcomp),sftdcx(npcmp),
     &        utidcx(npcmp),vtidcx(npcmp)
c
      save first,nocomp,dstart,t0tid,tptid,iomega,iphase,sftdcx,utidcx,
     &     vtidcx
c
      parameter (grav=980.6)
c
      data first /.true./
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
c-----------------------------------------------------------------------
c  On first call, read tidal data.
c-----------------------------------------------------------------------
c
      if (first) then
         if (tidfld.lt.2) then
            dstart=tstart
         else
            dstart=tidtstart
         endif
         call read_tide (dstart,im,jm,nocomp,iomega,iphase,t0tid,tptid,
     &                   sftdcx,utidcx,vtidcx)
         first = .false.
      end if
c
c-----------------------------------------------------------------------
c  Construct barotropic tidal fields.
c-----------------------------------------------------------------------
c
c  Set time, clear working arrays.
c
      ttsec = (day - tstart) * day2sec
c
      imjm = im*jm
c
      do 10 ip = 1, imjm
         ubtid(ip)  = c0
         vbtid(ip)  = c0
         elvtid(ip) = c0
         delvdt(ip) = c0
  10  continue
c
c  Compute barotropic tidal fields.
c
      do 20 n = 1, nocomp
         cxtime = CMPLX ( mod( (t0tid(n)+ttsec), tptid(n)) , c0 )
         e_iomt = exp(iomega(n)*cxtime + iphase(n))
         cshft = (n-1)*imjm
         do 20 ip = 1, imjm
            ipc = ip + cshft
            ut = ReaL(utidcx(ipc)*e_iomt)
            vt = ReaL(vtidcx(ipc)*e_iomt)
            et = ReaL(sftdcx(ipc)*e_iomt)
            ubtid(ip)  = ubtid(ip)  + ut
            vbtid(ip)  = vbtid(ip)  + vt
            elvtid(ip) = elvtid(ip) + et
            et = ReaL(iomega(n)*sftdcx(ipc)*e_iomt)
            delvdt(ip) = delvdt(ip) + et
  20  continue
c
c  Compute barotropic tidal surface pressure.
c
      do 30 ip = 1, im*jm
         pstid(ip) = elvtid(ip)*grav
  30  continue
c
      return
      end
