      subroutine heap_ikeysort (n,keyvec,tagvec)
c
c=======================================================================
c                                                                    ===
c  This routines sorts the key vector in ascending order using the   ===
c  Heapsort algorithm and makes the corresponding reordering in the  ===
c  second vector.  This code was adapted from Numerical Recipes.     ===
c                                                                    ===
c  ------                                                            ===
c  Input:                                                            ===
c  ------                                                            ===
c                                                                    ===
c  N........The number of elments in the vectors.   (integer)        ===
c  KEYVEC...The vector to be ordered.               (integer vector) ===
c  TAGVEC...The tag-along vector.                   (integer vector) ===
c                                                                    ===
c  -------                                                           ===
c  Output:                                                           ===
c  -------                                                           ===
c                                                                    ===
c  KEYVEC...The vector in ascending order.          (integer vector) ===
c  TAGVEC...The tag-along with same permutations.   (integer vector) ===
c                                                                    ===
c  ------                                                            ===
c  Calls:                                                            ===
c  ------                                                            ===
c                                                                    ===
c  none                                                              ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define local data.
c-----------------------------------------------------------------------
c
      integer i,ir,j,keyelm,l,n,tagelm
      integer keyvec(n),tagvec(n)
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
c-----------------------------------------------------------------------
c  Initialize indices for tree creation and sorting.  Protect the
c  trivial vector.
c-----------------------------------------------------------------------
c
      l = n/2 + 1
      ir = n
      if (ir .le. 1) then
         keyelm = keyvec(1)
         tagelm = tagvec(1)
      end if
c
      do 20 while (ir .gt. 1)
c
         if (l .gt. 1) then
c
c-----------------------------------------------------------------------
c           Still creating tree.
c-----------------------------------------------------------------------
c
            l = l - 1
            keyelm = keyvec(l)
            tagelm = tagvec(l)
c
           else
c
c-----------------------------------------------------------------------
c           Sorting tree.  Root of tree is largest element.  Clear it.
c-----------------------------------------------------------------------
c
            keyelm = keyvec(ir)
            tagelm = tagvec(ir)
            keyvec(ir) = keyvec(1)
            tagvec(ir) = tagvec(1)
            ir = ir - 1
c
         end if
c
c-----------------------------------------------------------------------
c        Sift element keyelm to its proper level.
c-----------------------------------------------------------------------
c
         i = l
         j = l + l
c
         do 10 while (j .le. ir)
c
            if (j .lt. ir) then
c
c-----------------------------------------------------------------------
c              Compare to greater branch.
c-----------------------------------------------------------------------
c
               if (keyvec(j) .lt. keyvec(j+1)) j = j + 1
c
            end if
c
            if (keyelm .lt. keyvec(j)) then
c
c-----------------------------------------------------------------------
c              Move keyelm deeper in tree.
c-----------------------------------------------------------------------
c
               keyvec(i) = keyvec(j)
               tagvec(i) = tagvec(j)
               i = j
               j = j + j
c
              else
c
c-----------------------------------------------------------------------
c              Found proper location, get out of loop.
c-----------------------------------------------------------------------
c
               j = ir + 1
c
            end if
c
 10      continue
c
c-----------------------------------------------------------------------
c        Put keyelm into its proper location.
c-----------------------------------------------------------------------
c
         keyvec(i) = keyelm
         tagvec(i) = tagelm
c
 20   continue
c
c-----------------------------------------------------------------------
c     Finished sorting, replace smallest element.
c-----------------------------------------------------------------------
c
      keyvec(1) = keyelm
      tagvec(1) = tagelm
c
      return
      end
