#ifdef coast
      subroutine rmmean_pbar (im,jm,lseg,pbar,isn,ien,dxt,tmetx,dyt,
     &                        tmety)
#else
      subroutine rmmean_pbar (im,jm,pbar,dxt,tmetx,dyt,tmety)
#endif
c
c=======================================================================
c                                                                    ===
c  This routine computes removes an integrated averaged value from   ===
c  the surface pressure.                                             ===
c                                                                    ===
c  ------                                                            ===
c  Input:                                                            ===
c  ------                                                            ===
c                                                                    ===
c     IM      Number grid points, X-direction.   (integer)           ===
c     JM      Number grid points, Y-direction.   (integer)           ===
#ifdef coast
c     LSEG    Maximum # active segments/row.     (integer)           ===
#endif
c     PBAR    Original surface pressure.         (real array)        ===
#ifdef coast
c     ISN     Row active starting indices.       (integer array)     ===
c     IEN     Row active ending indices.         (integer array)     ===
#endif
c     DXT     X-grid spacing on T-grid.          (real vector)       ===
c     TMETX   X-metric factor on T-grid.         (real array)        ===
c     DYT     Y-grid spacing on T-grid.          (real vector)       ===
c     TMETY   Y-metric factor on T-grid.         (real array)        ===
c                                                                    ===
c  -------                                                           ===
c  Output:                                                           ===
c  -------                                                           ===
c                                                                    ===
c     PBAR    Zero-mean surface pressure.        (real array)        ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define local data.
c-----------------------------------------------------------------------
c
      integer i,im,j,jm
#ifdef coast
      integer iend,istr,l,lseg
      integer isn(jm,lseg),ien(jm,lseg)
#endif
      real area,c0,c1,darea,pbarmean,rarea
      real dxt(im),dyt(jm),pbar(im,jm),tmetx(im,jm),tmety(im,jm)
c
      parameter (c0=0.0, c1=1.0)
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
c-----------------------------------------------------------------------
c  Compute physical area of averaging region.
c-----------------------------------------------------------------------
c
      area=c0
c
      do 20 j=2,jm-1
#ifdef coast
         do 20 l=1,lseg
            istr=isn(j,l)
            if(istr.eq.0) go to 20
            iend=ien(j,l)
            do 10 i=istr,iend
#else
            do 10 i=2,im-1
#endif
               darea    = dxt(i)*tmetx(i,j)*dyt(j)*tmety(i,j)
               area     = area + darea
  10        continue
  20  continue
c
      rarea = c1/area
c
c-----------------------------------------------------------------------
c  Compute interior mean value of surface pressure.
c-----------------------------------------------------------------------
c
      pbarmean=c0
c
      do 40 j=2,jm-1
#ifdef coast
         do 40 l=1,lseg
            istr=isn(j,l)
            if(istr.eq.0) go to 40
            iend=ien(j,l)
            do 30 i=istr,iend
#else
            do 30 i=2,im-1
#endif
               darea    = dxt(i)*tmetx(i,j)*dyt(j)*tmety(i,j)*rarea
               pbarmean = pbarmean + pbar(i,j)*darea
  30        continue
  40  continue
c
c-----------------------------------------------------------------------
c  Remove interior mean value of surface pressure.
c-----------------------------------------------------------------------
c
      do 60 j=1,jm
         do 50 i=1,im
            pbar(i,j) = pbar(i,j) - pbarmean
  50     continue
  60  continue
c
      return
      end
